# Generated by Django 4.0.9 on 2023-02-21 18:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taskmonitor", "0004_store_task_params_and_results"),
    ]

    operations = [
        migrations.AddField(
            model_name="tasklog",
            name="current_queue_length",
            field=models.IntegerField(
                default=None,
                help_text="Length of the queue at the time this log was created.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="tasklog",
            name="priority",
            field=models.IntegerField(
                db_index=True,
                default=None,
                help_text="Priority this task was executed with.",
                null=True,
            ),
        ),
    ]
