# Generated by Django 4.0.7 on 2022-09-06 15:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taskmonitor", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="TaskReport",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "report",
                "managed": False,
            },
        ),
    ]
